(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.rek = factory());
}(this, (function () { 'use strict';

  function FetchError(response, body) {
    this.name = 'FetchError';
    this.message = response.statusText;
    this.status = response.status;
    this.body = body;
    this.response = response;
    this.stack = new Error().stack;
  }

  var requestMethods = ['delete', 'get', 'head'];
  var dataMethods = ['patch', 'post', 'put'];
  var responseTypes = {
    arrayBuffer: '*/*',
    blob: '*/*',
    formData: 'multipart/form-data',
    json: 'application/json',
    text: 'text/*'
  };
  /**
   * @typedef {import('./types').Defaults} Defaults
   * @typedef {import('./types').Options} Options
   * @typedef {import('./types').Rek} Rek
   *
   * @param {Defaults} defaults
   * @returns {Rek}
   */

  function factory(defaults) {
    function rek(url, options) {
      if (typeof options === 'string') options = {
        response: options
      };
      options = Object.assign({}, defaults, options);

      if (options.baseUrl) {
        url = new URL(url, options.baseUrl).href;
      }

      if (options.searchParams) {
        url = url.split('?')[0] + '?' + new URLSearchParams(options.searchParams);
      }

      options.headers = new options.Headers(Object.assign({}, defaults.headers, options.headers));
      var _options = options,
          body = _options.body,
          headers = _options.headers,
          response = _options.response,
          fetch = _options.fetch;

      if (body && typeof body === 'object') {
        if (typeof body.append === 'function') {
          // is FormData or URLSearchParams
          headers.delete('content-type');
        } else if (typeof (body.getReader || body.stream) !== 'function' && (typeof body.byteLength !== 'number' || typeof (body.slice || body.getInt8) !== 'function')) {
          // `body` is not a valid `BodyInit` (needs to be ReadableStream
          // (.getReader), Blob (.stream), ArrayBuffer or DataView (.byteLength
          // and .slice or .getInt8)), stringifying
          options.body = JSON.stringify(body);
          headers.set('content-type', 'application/json');
        }
      }

      var onFullfilled = response && (typeof response === 'function' ? response : (headers.set('accept', responseTypes[response]), function (res) {
        return res.status === 204 ? null : res[response]();
      }));
      var res = fetch(url, options).then(function (res) {
        if (res.ok) return res;
        return res.text().then(function (text) {
          try {
            return JSON.parse(text);
          } catch (_unused) {
            return text || null;
          }
        }).catch(function () {
          return null;
        }).then(function (body) {
          throw new FetchError(res, body);
        });
      });
      return res.then(onFullfilled);
    }

    requestMethods.forEach(function (method) {
      rek[method] = function (url, options) {
        return rek(url, Object.assign({}, options, {
          method: method.toUpperCase()
        }));
      };
    });
    dataMethods.forEach(function (method) {
      rek[method] = function (url, body, options) {
        return rek(url, Object.assign({}, options, {
          body: body,
          method: method.toUpperCase()
        }));
      };
    });

    rek.extend = function (newDefaults) {
      return factory(Object.assign({}, defaults, newDefaults, {
        headers: Object.assign({}, defaults.headers, newDefaults.headers)
      }));
    };

    return rek;
  }

  var browser = factory({
    credentials: 'same-origin',
    response: 'json',
    fetch: fetch,
    Headers: Headers
  });
  /**
   * @typedef {import('./types').Defaults} Defaults
   * @typedef {import('./types').Options} Options
   * @typedef {import('./types').Rek} Rek
   */

  return browser;

})));
//# sourceMappingURL=rek.umd.es5.js.map
